﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using AngularAspNetMvc.Data.Models;

namespace AngularAspNetMvc.DataAccess.Reports
{
    public interface IReportsRunRepository
    {
        UserListDataSet GetUserListDataSet(IEnumerable<ReportRequestParameter> parameters);
        DataSet GetReportData(ReportRequest request);
    }
    public class ReportsRunRepository : Repository.Core.Repository, IReportsRunRepository
    {
        public UserListDataSet GetUserListDataSet(IEnumerable<ReportRequestParameter> parameters)
        {
            var reportRequestParameters = parameters as IList<ReportRequestParameter> ?? parameters.ToList();
            if (reportRequestParameters.Any())
            {
                var activeOnly = reportRequestParameters[0].ToString() == "1";
                var inactiveOnly = reportRequestParameters[0].ToString() == "0";
                var set = new UserListDataSet();
                var query = Context.Contacts.OrderBy(a => a.LastName);
                if (activeOnly)
                {
                    query = (IOrderedQueryable<Contact>)query.Where(a => a.IsActive);
                }
                if (inactiveOnly)
                {
                    query = (IOrderedQueryable<Contact>)query.Where(a => a.IsActive == false);
                }
                var data = from oneContact in query
                           select new
                           {
                               oneContact.ContactId,
                               oneContact.FirstName,
                               oneContact.LastName,
                               oneContact.ContactType.Name
                           };

                foreach (var oneContact in data)
                {
                    set.Contacts.AddContactsRow(
                        oneContact.FirstName, oneContact.LastName, oneContact.ContactId, oneContact.Name);
                }
                return set;
            }
            throw new ArgumentException("GetUserListDataSet received no parameters.");
        }

        public DataSet GetReportData(ReportRequest request)
        {
            if (request.ReportFileName == "ContactsList.rdlc")
            {
                return GetUserListDataSet(request.ReportRequestParameters);
            }
            throw new ArgumentException("GetReportData received invalid parameters.");
        }
    }
}