

module app.contacts.services {

    import IHttpResult = core.services.http.IHttpResult;
    import IContact = contacts.models.IContact;
    import IContactInfo = contacts.models.IContactInfo;
    import IPagedCriteria = common.models.IPagedCriteria;
    import IPagedResult = core.services.http.IPagedResult;



    export class ContactsService {
        getContacts(criteria: IPagedCriteria, callback: (result: IPagedResult<IContactInfo>) => void) {
            return this.http.get("/Contacts/Get", (pagedResult: IHttpResult<IPagedResult<IContactInfo>>) => {
                if (pagedResult.Success) {
                    callback(pagedResult.Result);
                }
            }, true, criteria);
        }
        getContact(id: number, callback: (result: IContact) => void) {
            return this.http.get("/Contacts/GetById", (result: IHttpResult<IContact>) => {
                if (result.Success) {
                    callback(result.Result);
                }
            }, true, { contactId: id });
        }
        deleteContact(contact: IContactInfo, callback: (result: boolean) => void) {
            return this.http.post("/Contacts/Delete", contact, (result: IHttpResult<boolean>) => {
                callback(result.Success);
            }, true);
        }
        saveContact(contact: IContact, callback: (result: IContact) => void) {
            return this.http.post("/Contacts/Save", contact, (result: IHttpResult<IContact>) => {
                if (result.Result) {
                    callback(result.Result);
                }
            }, true);
        }

        constructor(private http: core.services.http.IHttp) {
        }
    }

    angular.module("app.contacts.services", [])
        .service("contactsService", ["http", ContactsService]);
}