var app;
(function (app) {
    var directives;
    (function (directives) {
        var DirectiveHelper = (function () {
            function DirectiveHelper() {
            }
            DirectiveHelper.isEmpty = function (value) {
                return angular.isUndefined(value) || value === "" || value === null;
            };
            return DirectiveHelper;
        })();
        var RequiredDirective = (function () {
            function RequiredDirective() {
                this.restrict = "A";
                this.require = ["?ngModel"];
                this.link = function (scope, element, attributes, controller) {
                    var currentController = controller[0];
                    if (!currentController) {
                        return;
                    }
                    var validator = function (value) {
                        if ((DirectiveHelper.isEmpty(value) || value === false)) {
                            currentController.$setValidity("required", false);
                            return value;
                        }
                        else {
                            currentController.$setValidity("required", true);
                            return value;
                        }
                    };
                    currentController.$formatters.push(validator);
                    currentController.$parsers.unshift(validator);
                };
            }
            return RequiredDirective;
        })();
        var MaxLengthDirective = (function () {
            function MaxLengthDirective() {
                this.restrict = "A";
                this.require = ["?ngModel"];
                this.link = function (scope, element, attributes, controller) {
                    var maxLength = parseInt(element.attr(attributes.$attr.maxlen), 10);
                    var currentController = controller[0];
                    if (!currentController) {
                        return;
                    }
                    var validator = function (value) {
                        if (!DirectiveHelper.isEmpty(value) && value.length > maxLength) {
                            currentController.$setValidity("maxlen", false);
                            return value;
                        }
                        else {
                            currentController.$setValidity("maxlen", true);
                            return value;
                        }
                    };
                    currentController.$formatters.push(validator);
                    currentController.$parsers.unshift(validator);
                };
            }
            return MaxLengthDirective;
        })();
        var DropdownRequiredDirective = (function () {
            function DropdownRequiredDirective() {
                this.restrict = "A";
                this.require = ["?ngModel"];
                this.link = function (scope, element, attributes, controller) {
                    var currentController = controller[0];
                    if (!currentController) {
                        return;
                    }
                    var validator = function (value) {
                        if (DirectiveHelper.isEmpty(value)) {
                            currentController.$setValidity("dropdownRequired", false);
                            return value;
                        }
                        else {
                            var intValue = parseInt(value, 10);
                            if (intValue) {
                                currentController.$setValidity("dropdownRequired", true);
                            }
                            else {
                                currentController.$setValidity("dropdownRequired", false);
                            }
                            return value;
                        }
                    };
                    currentController.$formatters.push(validator);
                    currentController.$parsers.unshift(validator);
                };
            }
            return DropdownRequiredDirective;
        })();
        var FormSubmitDirectiveFunctionName = "formSubmitFunction";
        directives.FormSubmitDirectiveName = "formSubmit";
        var FormSubmitDirective = (function () {
            function FormSubmitDirective(utilities) {
                this.utilities = utilities;
                this.restrict = "A";
                this.link = function (scope, element, attributes) {
                    var $element = angular.element(element);
                    $element.bind("click", function (eventObject) {
                        eventObject.preventDefault();
                        var form = scope.$eval(attributes[directives.FormSubmitDirectiveName]);
                        // trigger all pristine fields to be dirty and run validation
                        angular.forEach(form, function (fieldController) {
                            if (fieldController && fieldController.$pristine) {
                                fieldController.$setDirty();
                            }
                        });
                        if (form.$valid) {
                            // call function defined in another attribute
                            scope.$eval(attributes[FormSubmitDirectiveFunctionName]);
                        }
                        else {
                            // apply scope changes to show error messages
                            scope.$apply();
                            utilities.showMessage("Please correct invalid values.");
                        }
                    });
                };
            }
            return FormSubmitDirective;
        })();
        angular.module("app.directives.validation", [])
            .directive("required", [function () {
                return new RequiredDirective();
            }])
            .directive("maxlen", [function () {
                return new MaxLengthDirective();
            }])
            .directive("dropdownRequired", [function () {
                return new DropdownRequiredDirective();
            }])
            .directive("formSubmit", ["utilities", function (utilities) {
                return new FormSubmitDirective(utilities);
            }]);
    })(directives = app.directives || (app.directives = {}));
})(app || (app = {}));
//# sourceMappingURL=validationDirectives.js.map