
module app.directives {

    class DirectiveHelper {
        static isEmpty(value: any) {
            return angular.isUndefined(value) || value === "" || value === null;
        }
    }

    class RequiredDirective implements ng.IDirective {
        restrict: string;
        require: string[];
        link: ng.IDirectiveLinkFn;
        constructor() {
            this.restrict = "A";
            this.require = ["?ngModel"];
            this.link = function (
                scope: core.controllers.ICoreScope, element: ng.IAugmentedJQuery, attributes: ng.IAttributes, controller: any) {

                var currentController: ng.INgModelController = controller[0];
                if (!currentController) {
                    return;
                }
                var validator = function (value) {
                    if ((DirectiveHelper.isEmpty(value) || value === false)) {
                        currentController.$setValidity("required", false);
                        return value;
                    } else {
                        currentController.$setValidity("required", true);
                        return value;
                    }
                };

                currentController.$formatters.push(validator);
                currentController.$parsers.unshift(validator);
            };
        }
    }

    interface IMaxLengthAttributes extends ng.IAttributes {
        $attr: IMaxLengthAttributesValues;
    }

    interface IMaxLengthAttributesValues {
        maxlen: string;
    }

    class MaxLengthDirective implements ng.IDirective {
        restrict: string;
        require: string[];
        link: ng.IDirectiveLinkFn;
        constructor() {
            this.restrict = "A";
            this.require = ["?ngModel"];
            this.link = function (
                scope: core.controllers.ICoreScope, element: ng.IAugmentedJQuery, attributes: IMaxLengthAttributes, controller: any) {
                var maxLength: number = parseInt(element.attr(attributes.$attr.maxlen), 10);
                var currentController: ng.INgModelController = controller[0];
                if (!currentController) {
                    return;
                }
                var validator = function (value) {
                    if (!DirectiveHelper.isEmpty(value) && value.length > maxLength) {
                        currentController.$setValidity("maxlen", false);
                        return value;
                    } else {
                        currentController.$setValidity("maxlen", true);
                        return value;
                    }
                };

                currentController.$formatters.push(validator);
                currentController.$parsers.unshift(validator);
            };
        }
    }

    class DropdownRequiredDirective implements ng.IDirective {
        restrict: string;
        require: string[];
        link: ng.IDirectiveLinkFn;
        constructor() {
            this.restrict = "A";
            this.require = ["?ngModel"];
            this.link = function (
                scope: core.controllers.ICoreScope, element: ng.IAugmentedJQuery, attributes: ng.IAttributes, controller: any) {
                var currentController: ng.INgModelController = controller[0];
                if (!currentController) {
                    return;
                }
                var validator = function (value) {
                    if (DirectiveHelper.isEmpty(value)) {
                        currentController.$setValidity("dropdownRequired", false);
                        return value;
                    } else {
                        var intValue = parseInt(value, 10);
                        if (intValue) {
                            currentController.$setValidity("dropdownRequired", true);
                        } else {
                            currentController.$setValidity("dropdownRequired", false);
                        }
                        return value;
                    }
                };

                currentController.$formatters.push(validator);
                currentController.$parsers.unshift(validator);
            };
        }
    }


    var FormSubmitDirectiveFunctionName = "formSubmitFunction";
    export var FormSubmitDirectiveName = "formSubmit";

    class FormSubmitDirective implements ng.IDirective {
        restrict: string;
        require: string[];
        link: ng.IDirectiveLinkFn;

        constructor(private utilities: core.services.IUtilities) {

            this.restrict = "A";
            this.link = function (scope: ng.IScope, element: ng.IAugmentedJQuery, attributes: ng.IAttributes) {
                var $element = angular.element(element);

                $element.bind("click", function (eventObject: JQueryEventObject) {
                    eventObject.preventDefault();

                    var form: ng.IFormController = scope.$eval(attributes[FormSubmitDirectiveName]);

                    // trigger all pristine fields to be dirty and run validation
                    angular.forEach(form, function (fieldController: ng.INgModelController) {
                        if (fieldController && fieldController.$pristine) {
                            fieldController.$setDirty();
                        }
                    });

                    if (form.$valid) {
                        // call function defined in another attribute
                        scope.$eval(attributes[FormSubmitDirectiveFunctionName]);
                    } else {
                        // apply scope changes to show error messages
                        scope.$apply();
                        utilities.showMessage("Please correct invalid values.");
                    }
                });
            };
        }
    }

    angular.module("app.directives.validation", [])
        .directive("required", [function () {
            return new RequiredDirective();
        }])
        .directive("maxlen", [function () {
            return new MaxLengthDirective();
        }])
        .directive("dropdownRequired", [function () {
            return new DropdownRequiredDirective();
        }])
        .directive("formSubmit", ["utilities", function (utilities: core.services.IUtilities) {
            return new FormSubmitDirective(utilities);
        }]);


}