/// <reference path="../../../scripts/typings/angularjs/angular.d.ts" />
/// <reference path="../../../app/home/interfaces.ts" />
/// <reference path="../../../scripts/typings/angularjs/angular-mocks.d.ts" />
describe("Contact Types Controller", function () {
    var controller: app.contacts.ContactsTypesController;
    var utilities: app.core.services.IUtilities;
    var $location: ng.ILocationService;
    var contactsTypeService: app.contactTypes.services.ContactsTypeService;
    var backEnd: ng.IHttpBackendService;

    beforeEach(function () {
        angular.mock.module("app.core.services.utilities");
        angular.mock.module("app.core.services.http");
        angular.mock.module("app.globalsModule");
        angular.mock.module("app.contactTypes.services");
    });

    beforeEach(inject(function (
        $rootScope: ng.IRootScopeService,
        _$location_: ng.ILocationService,
        _utilities_: app.core.services.IUtilities,
        _contactsTypeService_: app.contactTypes.services.ContactsTypeService,
        $httpBackend: ng.IHttpBackendService,
        globalsService: interfaces.IGLobals) {
        $location = _$location_;
        utilities = _utilities_;
        contactsTypeService = _contactsTypeService_;
        backEnd = $httpBackend;
        backEnd.when("GET", globalsService.webApiBaseUrl + "/ContactTypes/Get").respond(
            {
                "Success": true,
                "ErrorMessage": "",
                "Result": [
                    { "ContactTypeId": 2, "Name": "Coworker" },
                    { "ContactTypeId": 3, "Name": "Family" },
                    { "ContactTypeId": 5, "Name": "Fresh" },
                    { "ContactTypeId": 1, "Name": "Friend" }
                ]
            });
        backEnd.when("POST", globalsService.webApiBaseUrl + "/ContactTypes/Delete", { "ContactTypeId": 2, "Name": "Coworker" }).respond(
            {
                "Success": true,
                "ErrorMessage": "",
                "Result": { "ContactTypeId": 2, "Name": "Coworker" }
            });
    }));

    it("should create controller", () => {
        controller = new app.contacts.ContactsTypesController(utilities, $location, contactsTypeService);
        expect(controller).not.toBeNull();
        expect(typeof controller.editType).toBe("function");
        expect(typeof controller.deleteType).toBe("function");
    });

    it("should have data", () => {
        controller = new app.contacts.ContactsTypesController(utilities, $location, contactsTypeService);
        backEnd.flush();
        expect(controller.contactTypes).toBeDefined();
        expect(controller.contactTypes).not.toBeNull();
    });

    it("should have correct number of rows", () => {
        controller = new app.contacts.ContactsTypesController(utilities, $location, contactsTypeService);
        backEnd.flush();
        expect(controller.contactTypes.length).toBe(4);
    });

    it("should change location on edit", () => {
        controller = new app.contacts.ContactsTypesController(utilities, $location, contactsTypeService);
        backEnd.flush();
        expect(controller.contactTypes.length).toBe(4);
        controller.editType(controller.contactTypes[0]);
        expect($location.path()).toBe("/contacttype/edit/" + controller.contactTypes[0].ContactTypeId);
    });


    it("should change location on add", () => {
        controller = new app.contacts.ContactsTypesController(utilities, $location, contactsTypeService);
        backEnd.flush();
        expect(controller.contactTypes.length).toBe(4);
        controller.add();
        expect($location.path()).toBe("/contacttype/edit/add");
    });

    it("should delete type", () => {
        controller = new app.contacts.ContactsTypesController(utilities, $location, contactsTypeService);
        backEnd.flush();
        expect(controller.contactTypes.length).toBe(4);
        controller.deleteType(controller.contactTypes[0]);
        var deleteButton = $("#globalMessageDialog button").first();
        deleteButton.trigger("click");
        backEnd.flush();
        backEnd.verifyNoOutstandingExpectation();
        backEnd.verifyNoOutstandingRequest();
    });
});